/* nand-mn2ws-unit.h: unit-specific mtd file
 *
 * Copyright (C) 2006-2008 Panasonic Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 */
#ifndef _ASM_UNIT_NAND_MN2WS_UNIT_H
#define _ASM_UNIT_NAND_MN2WS_UNIT_H

#include <linux/config.h>

//#
//# Architecture dependent settings
//#

//Memory map of nand-gpbc
#define NAND_GPBC_UNIT_MEM_BASE		0x68000000
#define NAND_GPBC_UNIT_MEM_SIZE		0x00000020
#define NAND_GPBC_UNIT_REG_BASE		0x68100000
#define NAND_GPBC_UNIT_REG_SIZE		0x00001000

#ifdef	CONFIG_MTD_NAND_MN2WS_BB
  //the number of boot block {chip0, chip1, }
  #if defined(CONFIG_MTD_NAND_MAP_V2)
    /* add conditional coding in accordance with BUILDCONFIG_MODEL(K20/J20) (sharp 2014/01/15) */
    #if   defined(SH_CONFIG_BUILD_MODEL_K20)
    #define NAND_GPBC_UNIT_BTBLOCKS		{8, 0, 0, 0, 0, 0, 0, 0}	/* SDK V0011 or later */ //only backend boot
    #elif defined(SH_CONFIG_BUILD_MODEL_J20)
    #define NAND_GPBC_UNIT_BTBLOCKS		{16, 0, 0, 0, 0, 0, 0, 0}	/* based on SDK V0010 */ //only backend boot
    #else
    #error Build Model Unknown !!
    #endif
  #else
  #define NAND_GPBC_UNIT_BTBLOCKS		{16, 0, 0, 0, 0, 0, 0, 0} //with standby boot
  #endif


  //the number of maintain block {chip0, chip1, }
  #if defined(CONFIG_MTD_NAND_MAP_V2)
    #if (CONFIG_MTD_NAND_MN2WS_CHIP_BANK == 1)
    /* add conditional coding in accordance with BUILDCONFIG_MODEL(K20/J20) (sharp 2014/01/15) */
    #if   defined(SH_CONFIG_BUILD_MODEL_K20)
    #define NAND_GPBC_UNIT_MTBLOCKS		{64, 0, 0, 0, 0, 0, 0, 0}	/* SDK V0011 or later */
    #elif defined(SH_CONFIG_BUILD_MODEL_J20)
    #define NAND_GPBC_UNIT_MTBLOCKS		{128, 0, 0, 0, 0, 0, 0, 0}	/* based on SDK V0010 */
    #else
    #error Build Model Unknown !!
    #endif
    #elif (CONFIG_MTD_NAND_MN2WS_CHIP_BANK == 2)
    #define NAND_GPBC_UNIT_MTBLOCKS		{64, 64, 0, 0, 0, 0, 0, 0}
    #endif
  #else
    #error Unknown NAND MAP version !!
  #endif
#endif	//CONFIG_MTD_NAND_MN2WS_BB

// If maximum number of the corrected bit of each sector is larger than
// this definition, -EUCLEAN is notified to upper layer.
// If uncorrectable error occurs, -EBADMSG is notified to upper layer.
#define NAND_GPBC_CORRECTION_THRESHOLD	1


//#
//# Nand chip dependent settings
//#

// Enable banks
#define NAND_GPBC_FLASH_BANKS		(CONFIG_MTD_NAND_MN2WS_CHIP_BANK)

/* offset of ECC in oob area */
#define NAND_GPBC_UNIT_OOB_ECCPOS	{0} /* dummy. This parameter is not used. */

/* user part in oob area {offset, size} */
#define NAND_GPBC_ECCNUM \
	(CONFIG_MTD_NAND_MN2WS_CHIP_PAGESIZE / NAND_GPBC_ECCSIZE)
#define NAND_GPBC_ECC_TOTAL_SIZE \
	(NAND_GPBC_ECCBYTES * NAND_GPBC_ECCNUM)
#define NAND_GPBC_2ND_OOBFREE_OFFSET \
	(NAND_GPBC_OOB_SKIP_SIZE + NAND_GPBC_ECC_TOTAL_SIZE)
#define NAND_GPBC_2ND_OOBFREE_SIZE \
	(CONFIG_MTD_NAND_MN2WS_CHIP_OOBSIZE - NAND_GPBC_2ND_OOBFREE_OFFSET)

#ifdef CONFIG_MTD_NAND_MN2WS_USE_BBT_FLAG
#define NAND_GPBC_UNIT_OOB_OOBFREE \
	{{4,4}, {NAND_GPBC_2ND_OOBFREE_OFFSET,NAND_GPBC_2ND_OOBFREE_SIZE}}
#else /* CONFIG_MTD_NAND_MN2WS_USE_BBT_FLAG */
#define NAND_GPBC_UNIT_OOB_OOBFREE \
	{{2,6}, {NAND_GPBC_2ND_OOBFREE_OFFSET,NAND_GPBC_2ND_OOBFREE_SIZE}}
#endif /* CONFIG_MTD_NAND_MN2WS_USE_BBT_FLAG */

// pulse settings
#if defined(CONFIG_MTD_NAND_MN2WS_CHIP_TH58DVG4S0ETA20) || \
    defined(CONFIG_MTD_NAND_MN2WS_CHIP_TC58NVG2S0HTA00)
#define NAND_GPBC_UNIT_NCDWR	0x00001426
#define NAND_GPBC_UNIT_NCAAD	0x00001410
#define NAND_GPBC_UNIT_NRW		0x00000008
#define NAND_GPBC_UNIT_NRDCP	0x00000008
#define NAND_GPBC_UNIT_NRWLP	0x00000004
#define NAND_GPBC_UNIT_NRWHP	0x00000004
#define NAND_GPBC_UNIT_NCES		0x00000003
#define NAND_GPBC_UNIT_NRR		0x00000004
#define NAND_GPBC_UNIT_CA_BYTES	2
#define NAND_GPBC_UNIT_PA_BYTES	3

#elif defined(CONFIG_MTD_NAND_MN2WS_CHIP_TC58NVG3S0FTA00)
#define NAND_GPBC_UNIT_NCDWR	0x00001426
#define NAND_GPBC_UNIT_NCAAD	0x00001410
#define NAND_GPBC_UNIT_NRW		0x0000000E
#define NAND_GPBC_UNIT_NRDCP	0x00000008
#define NAND_GPBC_UNIT_NRWLP	0x00000004
#define NAND_GPBC_UNIT_NRWHP	0x00000004
#define NAND_GPBC_UNIT_NCES		0x00000003
#define NAND_GPBC_UNIT_NRR		0x00000004
#define NAND_GPBC_UNIT_CA_BYTES	2
#define NAND_GPBC_UNIT_PA_BYTES	3

#elif defined(CONFIG_MTD_NAND_MN2WS_CHIP_K9F2G08U0C)
#define NAND_GPBC_UNIT_NCDWR	0x00001426
#define NAND_GPBC_UNIT_NCAAD	0x00001410
#define NAND_GPBC_UNIT_NRW		0x00000016
#define NAND_GPBC_UNIT_NRDCP	0x00000008
#define NAND_GPBC_UNIT_NRWLP	0x00000005
#define NAND_GPBC_UNIT_NRWHP	0x00000004
#define NAND_GPBC_UNIT_NCES		0x00000003
#define NAND_GPBC_UNIT_NRR		0x00000004
#define NAND_GPBC_UNIT_CA_BYTES	2
#define NAND_GPBC_UNIT_PA_BYTES	3

#elif defined(CONFIG_MTD_NAND_MN2WS_CHIP_K9F1G08U0D_H27U1G8F2CTR_TC58NVG0S3ETA00)
#define NAND_GPBC_UNIT_NCDWR	0x0000140e
#define NAND_GPBC_UNIT_NCAAD	0x00001410
#define NAND_GPBC_UNIT_NRW	0x00000016
#define NAND_GPBC_UNIT_NRDCP	0x00000008
#define NAND_GPBC_UNIT_NRWLP	0x00000005
#define NAND_GPBC_UNIT_NRWHP	0x00000003
#define NAND_GPBC_UNIT_NCES	0x00000003
#define NAND_GPBC_UNIT_NRR	0x00000004
#define NAND_GPBC_UNIT_CA_BYTES	2
#define NAND_GPBC_UNIT_PA_BYTES	2

#elif defined(CONFIG_MTD_NAND_MN2WS_CHIP_H27U4G8F2D) || \
      defined(CONFIG_MTD_NAND_MN2WS_CHIP_H27U2G8F2CTR_TC58NVG1S3ETA00B3H_S34ML02G100) || \
      defined(CONFIG_MTD_NAND_MN2WS_CHIP_TC58NVG1S3HTA00)
#define NAND_GPBC_UNIT_NCDWR	0x00001426
#define NAND_GPBC_UNIT_NCAAD	0x00001410
#define NAND_GPBC_UNIT_NRW		0x00000008
#define NAND_GPBC_UNIT_NRDCP	0x00000009
#define NAND_GPBC_UNIT_NRWLP	0x00000007
#define NAND_GPBC_UNIT_NRWHP	0x00000004
#define NAND_GPBC_UNIT_NCES		0x00000003
#define NAND_GPBC_UNIT_NRR		0x00000004
#define NAND_GPBC_UNIT_CA_BYTES	2
#define NAND_GPBC_UNIT_PA_BYTES	3

#elif defined(CONFIG_MTD_NAND_MN2WS_CHIP_S34ML04G1)		/* Share value with S34ML04G2 */
#define NAND_GPBC_UNIT_NCDWR	0x0000142a
#define NAND_GPBC_UNIT_NCAAD	0x0000140a
#define NAND_GPBC_UNIT_NRW	0x00000016
#define NAND_GPBC_UNIT_NRDCP	0x00000008
#define NAND_GPBC_UNIT_NRWLP	0x00000005
#define NAND_GPBC_UNIT_NRWHP	0x00000003
#define NAND_GPBC_UNIT_NCES	0x00000003
#define NAND_GPBC_UNIT_NRR	0x00000004
#define NAND_GPBC_UNIT_CA_BYTES	2
#define NAND_GPBC_UNIT_PA_BYTES	3

#elif defined(CONFIG_MTD_NAND_MN2WS_CHIP_MX30LF1208AA) || defined(CONFIG_MTD_NAND_MN2WS_CHIP_MX30LF1G08AA)
#define NAND_GPBC_UNIT_NCDWR	0x0000140e
#define NAND_GPBC_UNIT_NCAAD	0x00001410
#define NAND_GPBC_UNIT_NRW	0x00000002
#define NAND_GPBC_UNIT_NRDCP	0x00000008
#define NAND_GPBC_UNIT_NRWLP	0x00000006
#define NAND_GPBC_UNIT_NRWHP	0x00000004
#define NAND_GPBC_UNIT_NCES	0x00000003
#define NAND_GPBC_UNIT_NRR	0x00000004
#define NAND_GPBC_UNIT_CA_BYTES	2
#define NAND_GPBC_UNIT_PA_BYTES	2

#else
#error Unknown NAND pulse setting !!
#endif

//#
//# Partition settings
//#

/* NOTICE: Partition setting has been moved to unit-init.c */

#endif	//_ASM_UNIT_NAND_MN2WS_UNIT_H
